﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.ExpressionGenerator {
  public class ExpressionEvaluator {
    private readonly Dictionary<Expression, double> evaluationCache;

    public ExpressionEvaluator() {
      evaluationCache = new Dictionary<Expression, double>();
    }

    public double Evaluate(Expression expression) {
      double evaluation;
      if (evaluationCache.TryGetValue(expression, out evaluation))
        return evaluation;

      switch (expression.Type) {
        case ExpressionType.Constant:
          evaluation = expression.Value;
          break;
        case ExpressionType.RandomVariable:
          evaluation = expression.Distribution.NextDouble();
          break;
        case ExpressionType.Function:
          var args = expression.Arguments.Select(Evaluate);
          evaluation = expression.Transform(args);
          break;
        default:
          throw new ArgumentException("Unknown expression type.");
      }

      evaluationCache[expression] = evaluation;
      return evaluation;
    }

    public Dictionary<Expression, List<double>> GenerateData(Expression expression, int rows) {
      return GenerateData(new [] {expression}, rows);
    }
    public Dictionary<Expression, List<double>> GenerateData(IEnumerable<Expression> expressions, int rows) {
      var data = new Dictionary<Expression, List<double>>();
      for (int i = 0; i < rows; ++i) {
        evaluationCache.Clear();
        foreach(var expr in expressions) Evaluate(expr);
        foreach (var pair in evaluationCache) {
          if (!data.ContainsKey(pair.Key))
            data[pair.Key] = new List<double>();

          data[pair.Key].Add(pair.Value);
        }
      }
      return data;
    }
  }
}
